#ifndef UTIL_H
#include "util.h"
#endif


/*
--------------------------------------------------------------------------------
                                  CONSTRUCTOR
--------------------------------------------------------------------------------
*/
Ball::Ball( void )
{
    /*  Init offsets */
    mPos.x = 0;
    mPos.y = 0;

    /*  Set collision box dimensions */
    mCollider.w = mWidth;
    mCollider.h = mHeight;

    /*  Init velocity */
    mMaxVelocity = 12;
    mVelX = 8;
    mVelY = 8;
    mVelocityDefault = 8;

    /*  Init directions */
    dx = 0;
    dy = 0;

    /*  Init hits */
    mHits = 0;

    /*  Max bounce angle */
    mMaxBounceAngle = 90;

}


/*
--------------------------------------------------------------------------------
                                      HIT

    !-- NOTE:  This class method isn't actually used in the final 'game', but
        I'm leaving it here just in case I change my mind about that in the
        future.
--------------------------------------------------------------------------------
*/
void Ball::hit( void )
{
    /*  Increment the number of times the ball has been hit */
    ++mHits;

    /*  Every four hits, increase X velocity */
    if( mHits % 4 == 0 )
    {
        //  Reset hit count just to keep it from going crazy high
        mHits = 0;

        if( mVelX < mMaxVelocity )
        {
            ++mVelX;
            ++dx;
        }
    }
}


/*
--------------------------------------------------------------------------------
                                     UPDATE
--------------------------------------------------------------------------------
*/
void Ball::update( void )
{
    /*  Check for wall boundaries */

    //  Left wall
    if( mPos.x < 0 )
    {
        /*  Increase right paddle score */
        gScores[ 1 ]->set_score( gScores[ 1 ]->get_score() + 1 );

        /*  Play the sound */
        if( ! ( Mix_Playing( 0 ) > 0) && ! mute )
        {
            Mix_PlayChannel( 0, gSoundScore, 0 );
        }

        /*  Reset the position */
        mPos.x = ( SCREEN_WIDTH - mWidth ) / 2;
        mPos.y = ( SCREEN_HEIGHT - mHeight ) / 2;

        /*  Flip the X direction */
        dx *= -1;

        /*  Pause the game */
        paused = true;
        reset = true;

        /*  Reset ball velocity */
        mVelX = 4;
        mVelY = 4;
    }

    //  Right wall
    if( mPos.x + mWidth > SCREEN_WIDTH )
    {

        /*  Increase left paddle score */
        gScores[ 0 ]->set_score( gScores[ 0 ]->get_score() + 1 );

        /*  Play the sound */
        if( ! ( Mix_Playing( 0 ) > 0) && ! mute )
        {
            Mix_PlayChannel( 0, gSoundScore, 0 );
        }

        /*  Reset the position */
        mPos.x = ( SCREEN_WIDTH - mWidth ) / 2;
        mPos.y = ( SCREEN_HEIGHT - mHeight ) / 2;

        /*  Flip the X direction */
        dx *= -1;

        /*  Pause the game */
        paused = true;
        reset = true;

        /*  Reset ball velocity */
        mVelX = 4;
        mVelY = 4;
    }

    //  Ceiling
    if( mPos.y < 0 )
    {
        set_position( mPos.x, 0 );
        reset_position();
        flip_dy();

        if( ! ( Mix_Playing( 1 ) > 0 ) && ! mute )
        {
            Mix_PlayChannel( 1, gSoundBounceWall, 0 );
        }
    }

    //  Floor
    if( mPos.y + mHeight > SCREEN_HEIGHT )
    {
        set_position( mPos.x, SCREEN_HEIGHT - mHeight );
        reset_position();
        flip_dy();

        if( ! ( Mix_Playing( 1 ) > 0 ) && ! mute )
        {
            Mix_PlayChannel( 1, gSoundBounceWall, 0 );
        }
    }

}


/*
--------------------------------------------------------------------------------
                                      MOVE
--------------------------------------------------------------------------------
*/
void Ball::move( void )
{
    /*  Modify X and Y positions according to X and Y direction */
    mPos.x += dx;
    mPos.y += dy;

    /*  Set the collider equal to the X and Y position */
    mCollider.x = mPos.x;
    mCollider.y = mPos.y;

    /*  Handle collision */
    handle_collision();

    /*  Update the ball/boundaries situation */
    update();
}


/*
--------------------------------------------------------------------------------
                                 RESET_POSITION
--------------------------------------------------------------------------------
*/
void Ball::reset_position( void )
{
    /*  Move the ball back */
    mPos.x -= dx;
    mPos.y -= dy;

    /*  Same for collider */
    mCollider.x = mPos.x;
    mCollider.y = mPos.y;
}


/*
--------------------------------------------------------------------------------
                                   COLLISION
--------------------------------------------------------------------------------
*/
bool Ball::collision( SDL_Rect paddle )
{
    /*
     *  Separating axis test
     */


    /*  Calculate the sides */
    int leftA, leftB;
    int rightA, rightB;
    int topA, topB;
    int bottomA, bottomB;

    /*  Rectangle A */
    leftA = mCollider.x;
    rightA = mCollider.x + mCollider.w;
    topA = mCollider.y;
    bottomA = mCollider.y + mCollider.h;

    /*  Rectangle B */
    leftB = paddle.x;
    rightB = paddle.x + paddle.w;
    topB = paddle.y;
    bottomB = paddle.y + paddle.h;

    if( bottomA <= topB )
        return( false );
    if( topA >= bottomB )
        return( false );
    if( rightA <= leftB )
        return( false );
    if( leftA >= rightB )
        return( false );

    /*  If none of those were true:  Collision */
    return( true );
}


/*
--------------------------------------------------------------------------------
                             SET AND GET DIMENSIONS
--------------------------------------------------------------------------------
*/
void Ball::set_dx( int dX )
{
    dx = dX;
}


void Ball::set_dy( int dY )
{
    dy = dY;
}


int Ball::get_dx( void )
{
    return( dx );
}


int Ball::get_dy( void )
{
    return( dy );
}


void Ball::flip_dx( void )
{
    dx *= -1;
}


void Ball::flip_dy( void )
{
    dy *= -1;
}


/*
--------------------------------------------------------------------------------
                                BALL ANGLE STUFF
--------------------------------------------------------------------------------
*/
void Ball::set_max_bounce_angle( int angle )
{
    /*  We can set the angle to whatever feels good here */
    if( angle < 360 && angle > 0 )
        mMaxBounceAngle = angle;
}


int Ball::get_max_bounce_angle( void )
{
    return( mMaxBounceAngle );
}
