#ifndef UTIL_H
#include "util.h"
#endif


/*
--------------------------------------------------------------------------------
                                  CONSTRUCTOR
--------------------------------------------------------------------------------
*/
RectThing::RectThing( void )
{
    /*  Init offsets */
    mPos.x = 0;
    mPos.y = 0;

    /*  Init dimensions */
    mWidth = 0;
    mHeight = 0;

    /*  Init velocity */
    mMaxVelocity = 0;
    mVelX = 0;
    mVelY = 0;
    mVelocityBleeding = false;

    /*  Init moveticks */
    moveTicks = 0;

    /*  Init difficulty of paddles */
    mDifficulty = 1;
    mMaxDifficulty = 5;
}


/*
--------------------------------------------------------------------------------
                                    POSITION
--------------------------------------------------------------------------------
*/
void RectThing::set_position( int x, int y )    //  Set X and Y position
{
    mPos.x = x;
    mPos.y = y;
}

int RectThing::get_pos_x( void )                //  Get X position
{
    return( mPos.x );
}

int RectThing::get_pos_y( void )                //  Get Y position
{
    return( mPos.y );
}


/*
--------------------------------------------------------------------------------
                                   DIMENSIONS
--------------------------------------------------------------------------------
*/
void RectThing::set_width( int width )          //  Set width
{
    mWidth = width;
}

int RectThing::get_width( void )                //  Get width
{
    return( mWidth );
}

void RectThing::set_height( int height )        //  Set height
{
    mHeight = height;
}

int RectThing::get_height( void )               //  Get height
{
    return( mHeight );
}



/*
--------------------------------------------------------------------------------
                                    COLLIDER
--------------------------------------------------------------------------------
*/
SDL_Rect RectThing::get_collider( void )        //  Get collider (SDL_Rect)
{
    return( mCollider );
}

void RectThing::init_collider( void )           //  Initialize collider
{
    mCollider.x = mPos.x;
    mCollider.y = mPos.y;
    mCollider.w = mWidth;
    mCollider.h = mHeight;
}



/*
--------------------------------------------------------------------------------
                                     RENDER
--------------------------------------------------------------------------------
*/

/*  Copy filled rect to the renderer */
void RectThing::render( void )
{
    /*  Set the draw rectangle to the object's position and dimensions */
    SDL_Rect drawRect = { mPos.x, mPos.y, mWidth, mHeight };

    /*  Set the draw color to the foreground color at full opacity */
    SDL_SetRenderDrawColor( gRenderer, fgColor.r, fgColor.g, fgColor.b, 0xff);

    /*  Draw the filled rect */
    SDL_RenderFillRect( gRenderer, &drawRect );

    /*  If we want to outline, draw the outline on top of the filled rect */
    if( outline )
        draw_outline();
}


/*  Draw the outline for this rectangle object */
void RectThing::draw_outline( void )
{
    /*  Set the draw rect to the position and dimensions of this object */
    SDL_Rect drawRect = { mPos.x, mPos.y, mWidth, mHeight };

    /*  Set the render draw color to the outline color at full opacity */
    SDL_SetRenderDrawColor( gRenderer, outlineColor.r,
            outlineColor.g, outlineColor.b, 255 );

    /*  Draw the outline rect */
    SDL_RenderDrawRect( gRenderer, &drawRect );

}



/*
--------------------------------------------------------------------------------
                                    VELOCITY
--------------------------------------------------------------------------------
*/
void RectThing::set_velocity( int xVel, int yVel )  //  Set X and Y velocity
{
    mVelX = xVel;
    mVelY = yVel;
}

void RectThing::get_velocity( int *vX, int *vY )    //  Get X and Y velocity
{
    *vX = mVelX;
    *vY = mVelY;
}


int RectThing::get_velocity_x( void )               //  Get X velocity
{
    return( mVelX );
}


int RectThing::get_velocity_y( void )               //  Get Y velocity
{
    return( mVelY );
}

void RectThing::set_max_velocity( int velocity )    //  Set maximum velocity
{
    mMaxVelocity = velocity;
}



/*
--------------------------------------------------------------------------------
                                   DIFFICULTY
--------------------------------------------------------------------------------
*/
void RectThing::increase_difficulty( void )         //  Increase difficulty
{
    ++mDifficulty;
    if( mDifficulty > mMaxDifficulty )
        mDifficulty = 1;
}


void RectThing::decrease_difficulty( void )         //  Decrease difficulty
{
    --mDifficulty;
    if( mDifficulty < 1 )
        mDifficulty = mMaxDifficulty;
}


void RectThing::set_difficulty( int difficulty )    //  Set difficulty
{
    if( difficulty > mMaxDifficulty )
        mDifficulty = mMaxDifficulty;
    else if( difficulty < 1 )
        mDifficulty = 1;
    else
        mDifficulty = difficulty;
}

